/*
 * $Id: AlarmSet.cpp,v 1.3 2002/10/06 22:26:20 dwalters Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * Modified by Anton Maslovsky <my-zaurus@narod.ru> 2003/01/28
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include <qcombobox.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qgroupbox.h>

#include <qpe/applnk.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/timestring.h>
#include <qpe/fileselector.h>
#include <qpe/resource.h>

#include "AlarmSet.h"
#include "AlarmMessage.h"
#include "qpealarmclock.h"
#include "sharp_char.h"

AlarmSet::AlarmSet(AlarmDetails* alarmDetailsInit, QWidget* parent, 
		   const char* name, bool modal, WFlags fl) : 
                   AlarmSetBase(parent, name, modal, fl), 
                   alarmDetails(alarmDetailsInit), selector(NULL)
{
    groupTime->setOrientation(Qt::Horizontal);
    groupTime->setAlignment(Qt::AlignLeft);
        
    // fixup sizes
    QImage imgAlarm = Resource::loadImage("qpealarmclock/QpeAlarmClock");
    QPixmap pixAlarm;
    pixAlarm.convertFromImage(imgAlarm.smoothScale(AppLnk::smallIconSize(), AppLnk::smallIconSize()));
    
    PixmapLabel3->setPixmap(pixAlarm);
    PixmapLabel3->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    PixmapLabel3->setMaximumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());

    lblColon->setMaximumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    spinHour->setMaximumSize(AppLnk::smallIconSize() * 3, AppLnk::smallIconSize() * 2);
    spinMinute->setMaximumSize(AppLnk::smallIconSize() * 3, AppLnk::smallIconSize() * 2);

    QImage imgFrequency = Resource::loadImage("qpealarmclock/frequency");
    QPixmap pixFrequency;
    pixFrequency.convertFromImage(imgFrequency.smoothScale(AppLnk::smallIconSize(), AppLnk::smallIconSize()));
    
    PixmapLabel2->setPixmap(pixFrequency);
    PixmapLabel2->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    
    grpSnooze->setOrientation(Qt::Horizontal);
    grpSnooze->setAlignment(Qt::AlignLeft);

    PixmapLabel5->setPixmap(pixAlarm);
    PixmapLabel5->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    PixmapLabel5->setMaximumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    
    spinSnooze->setMaximumSize(AppLnk::smallIconSize() * 3, AppLnk::smallIconSize() * 2);
    
    for (int i = AlarmDetails::once; i <= AlarmDetails::custom; i++)
      cboFrequency->insertItem(AlarmDetails::frequencyLabel(static_cast<AlarmDetails::Frequency>(i)));

    connect(cboFrequency, SIGNAL(activated(int)), this, SLOT(cboFrequencyActivated(int)));
    cboFrequency->setCurrentItem(alarmDetails->frequency());

    // update week days order
    if (((AlarmClock*)parent)->weekStartMonday)
    {
        lblSun->hide();
        chkSun->hide();
        
        Layout5->add(lblSun);
        Layout1->add(chkSun);

        Layout1->invalidate();
        Layout5->invalidate();
        
        groupFrequency->updateGeometry();

        lblSun->show();
        chkSun->show();
        
        updateGeometry();
    }
    

    // populate checkboxes
    chkWeekdays[0] = chkMon;
    chkWeekdays[1] = chkTue;
    chkWeekdays[2] = chkWed;
    chkWeekdays[3] = chkThu;
    chkWeekdays[4] = chkFri;
    chkWeekdays[5] = chkSat;
    chkWeekdays[6] = chkSun;

    setWeekdaysFlags(alarmDetails->weekdaysFlags(), alarmDetails->frequency() == AlarmDetails::custom);

    cboNotification->insertItem(AlarmDetails::notificationLabel(AlarmDetails::silent));
    cboNotification->insertItem(AlarmDetails::notificationLabel(AlarmDetails::loud));
    cboNotification->setCurrentItem(alarmDetails->notification());

    txtAlarmSound->setText(alarmDetails->sound());
    QListBoxItem* item = cmbDevice->listBox()->findItem(alarmDetails->device());
    if (item != NULL)
      cmbDevice->setCurrentItem(cmbDevice->listBox()->index(item));

    txtAlarmMessage->setText(alarmDetails->message());

    // always enable alarm when user starts to modify it
    chkAlarmEnabled->setChecked(true /* alarmDetails->enabled() */);

    for (int i = LED_COLLIE_1_DEFAULT; i <= LED_COLLIE_1_SOFTFLASH; i++)
      cboLed->insertItem(AlarmDetails::ledLabel(i));
    cboLed->setCurrentItem(alarmDetails->led());
    
    alarmTime = alarmDetails->time();
    
    spinMinute->setValue(alarmTime.minute());
    if (alarmTime.minute() < 10)
      spinMinute->setPrefix("0");
   
    if (((AlarmClock*)parent)->twelveHourClock)
    {
      spinHour->setMinValue(1);
      spinHour->setMaxValue(12);
      spinHour->setValue((alarmTime.hour() % 12 == 0) ? 12 : 
                         alarmTime.hour() < 12 ? alarmTime.hour() : 
                         alarmTime.hour() - 12);
      if (alarmTime.hour() >= 12)
        chkPM->setChecked(true);
    }
    else
    {
      spinHour->setMinValue(0);
      spinHour->setMaxValue(23);
      spinHour->setValue(alarmTime.hour());
      chkPM->hide();
    }

    connect(spinHour, SIGNAL(valueChanged(int)), this, 
            SLOT(hourChanged(int)));
    connect(spinMinute, SIGNAL(valueChanged(int)), this, 
            SLOT(minuteChanged(int)));
    connect(chkPM, SIGNAL(stateChanged(int)), this, 
            SLOT(chkPMChanged(int)));

    spinSnooze->setValue(alarmDetailsInit->snoozeInterval());

    connect(btnSelectWavFile, SIGNAL(clicked()), this, SLOT(selectWavFileClicked()));
    btnSelectWavFile->setPixmap(Resource::loadPixmap("volume"));
    
    connect(btnPlay, SIGNAL(clicked()), this, SLOT(playFileClicked()));
    btnPlay->setPixmap(Resource::loadPixmap("play"));
    
    connect(btnPreview, SIGNAL(clicked()), this, SLOT(previewClicked()));

    frmFiles->hide();
}

AlarmSet::~AlarmSet()
{
}

void AlarmSet::chkPMChanged(int state)
{
    if (((AlarmClock*)parent())->twelveHourClock)
    {
        int hour;
        if (state)
        {
            // PM
            hour = (spinHour->value() % 12 == 0) ? 12 : 
                    spinHour->value() + 12;
        }
        else
        {
            // AM
            hour = (spinHour->value() % 12 == 0) ? 0 : 
                    spinHour->value();
        }
        alarmTime = QTime(hour, alarmTime.minute());
    }
}

void AlarmSet::hourChanged(int value)
{
    int hour;
    if (((AlarmClock*)parent())->twelveHourClock)
    {
        if (chkPM->isChecked())
        {
            // PM
            hour = (value % 12 == 0) ? 12 : value + 12;
        }
        else
        {
            // AM
            hour = (value % 12 == 0) ? 0 : value;
        }
    }
    else
    {
        hour = value;
        if (value < 10)
            spinHour->setPrefix("0");
        else
            spinHour->setPrefix("");
    }

    alarmTime = QTime(hour, alarmTime.minute());
}

void AlarmSet::minuteChanged(int value)
{
    alarmTime = QTime(alarmTime.hour(), value);
    if (value < 10)
        spinMinute->setPrefix("0");
    else
        spinMinute->setPrefix("");
}

void AlarmSet::cboFrequencyActivated(int item)
{
  setWeekdaysFlags(AlarmDetails::weekdaysGroups[item], item == AlarmDetails::custom);
}

void AlarmSet::setWeekdaysFlags(AlarmDetails::Weekdays weekdays, bool enable /* = false */)
{
  for (int i = 0; i < NUM_WEEKDAYS; i++)
  {
    if (weekdays != AlarmDetails::WEEKDAYS_IGNORE)
      chkWeekdays[i]->setChecked(weekdays & AlarmDetails::flags[i]);

    chkWeekdays[i]->setEnabled(enable);
  }
}

AlarmDetails::Weekdays AlarmSet::weekdaysFlags() const
{
  unsigned result = AlarmDetails::WEEKDAYS_NONE;
  
  for (int i = 0; i < NUM_WEEKDAYS; i++)
  {
    if (chkWeekdays[i]->isChecked())
      result |= AlarmDetails::flags[i];
  }

  return static_cast<AlarmDetails::Weekdays>(result);
}

void AlarmSet::accept()
{
    alarmDetails->setAlarm(chkAlarmEnabled->isChecked(), alarmTime,
      static_cast<AlarmDetails::Frequency>(cboFrequency->currentItem()),
      txtAlarmMessage->text(),
      static_cast<AlarmDetails::Notification>(cboNotification->currentItem()),
      cboLed->currentItem(),
      txtAlarmSound->text(),
      cmbDevice->currentText(),
      spinSnooze->value(),
      weekdaysFlags());

	QCopEnvelope e("QPE/clockApplet", "alarmsChanged()");
    AlarmSetBase::accept();
}

void AlarmSet::selectWavFileClicked()
{
  if (!frmFiles->isVisible())
  {
    if (selector == NULL)
    {
      QVBoxLayout* vb = new QVBoxLayout(frmFiles);
      selector = new FileSelector("audio/x-wav", frmFiles, 0, FALSE, FALSE);
      selector->setMaximumHeight(frmFiles->maximumHeight());
      vb->add(selector);

      connect(selector, SIGNAL(fileSelected(const DocLnk& )), this, SLOT(fileSelected(const DocLnk&)));
    }  

    frmFiles->show();
  }
  else
    frmFiles->hide();
}

void AlarmSet::fileSelected(const DocLnk& link)
{
    txtAlarmSound->setText(link.file());
}

void AlarmSet::playFileClicked()
{
    AlarmMessage::playSound(cmbDevice->currentText(), txtAlarmSound->text()); 
}

void AlarmSet::previewClicked()
{
    AlarmDetails tmpAlarm(99);
    tmpAlarm.setAlarm(chkAlarmEnabled->isChecked(), 
        alarmTime,
        static_cast<AlarmDetails::Frequency>(cboFrequency->currentItem()),
        txtAlarmMessage->text(),
        static_cast<AlarmDetails::Notification>(cboNotification->currentItem()),
        cboLed->currentItem(),
        txtAlarmSound->text(),
        cmbDevice->currentText(),
        spinSnooze->value(),
        weekdaysFlags());
    
    AlarmMessage dlg(&tmpAlarm, this, "Alarm", true);
    dlg.showMaximized();
    dlg.exec();
}
