/*
 * $Id: qpealarmclock.cpp,v 1.4 2002/11/17 06:11:43 dwalters Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * Modified by Anton Maslovsky <my-zaurus@narod.ru> 2003/01/28
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include <qpe/config.h>
#include <qpe/qpeapplication.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>

#include <qimage.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qgroupbox.h>
#include <qapplication.h>

#include "qpealarmclock.h"
#include "AlarmSet.h"
#include "AlarmMessage.h"


AlarmClock::AlarmClock( QWidget* parent,  const char* name, WFlags fl )
    : AlarmClockBase( parent, name, fl )
{
    // fixup sizes
    QWidget* d = QApplication::desktop();
    
    QImage imgAlarm = Resource::loadImage("qpealarmclock/QpeAlarmClock");
    QPixmap pixAlarm;
    pixAlarm.convertFromImage(imgAlarm.smoothScale(AppLnk::smallIconSize(), AppLnk::smallIconSize()));
    
    pixIcon->setPixmap(pixAlarm);
    pixIcon->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    
    btnAlarm1->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    btnAlarm2->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    btnAlarm3->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    btnAlarm4->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    
    if (d->width() > 240)
    {
        lblTime1->setMaximumSize(80, 32767);
        lblTime2->setMaximumSize(80, 32767);
        lblTime3->setMaximumSize(80, 32767);
        lblTime4->setMaximumSize(80, 32767);
    }
    
    btnAbout->setMinimumSize(AppLnk::smallIconSize(), AppLnk::smallIconSize());
    
    ////
    
    timer = new QTimer(this, "timer");
    timer->start(2000);
    
    connect(btnAbout, SIGNAL(clicked()), this, SLOT(aboutClicked()));
    connect(btnAlarm1, SIGNAL(clicked()), this, SLOT(alarm1clicked()));
    connect(btnAlarm2, SIGNAL(clicked()), this, SLOT(alarm2clicked()));
    connect(btnAlarm3, SIGNAL(clicked()), this, SLOT(alarm3clicked()));
    connect(btnAlarm4, SIGNAL(clicked()), this, SLOT(alarm4clicked()));
    connect(timer, SIGNAL(timeout()), this, SLOT(timerTriggered()));

	/*
    QCopChannel* chan = new QCopChannel("QPE/Application/qpealarmclock", this );
    connect(chan, SIGNAL(received(const QCString&,const QByteArray&) ),
            this, SLOT(slotMessage(const QCString&, const QByteArray& ) ) );
	*/

    connect( qApp, SIGNAL(appMessage(const QCString&, const QByteArray&)),
	     this, SLOT(appMessage(const QCString&, const QByteArray&)) );

	// when the last window is closed, the application should quit
    connect( qApp, SIGNAL( lastWindowClosed() ), qApp, SLOT( quit() ) );

    initTwelveHourClock();

	loadAlarms();
    updateUI();
}

AlarmClock::~AlarmClock()
{	
	clearAlarms();
    delete timer;
}

void AlarmClock::loadAlarms()
{
    for (int i = 0; i < NUM_ALARMS; i++)
      alarms[i] = new AlarmDetails(i);
}

void AlarmClock::clearAlarms()
{
    for (int i = 0; i < NUM_ALARMS; i++)
      delete alarms[i];
}

void AlarmClock::timerTriggered()
{
    // Calculate time to next alarm
    QDateTime nextAlarm;
    QString strNextAlarm;
    for (int i = 0; i < NUM_ALARMS; i++)
    {
      if (alarms[i]->enabled() &&
          alarms[i]->nextDateTime().isValid() && (
          (!nextAlarm.isValid()) || (alarms[i]->nextDateTime() < nextAlarm)))
      {
        nextAlarm = alarms[i]->nextDateTime();
      }
    }

    if (nextAlarm.isValid())
    {
      int tot_secs = QDateTime::currentDateTime().secsTo(nextAlarm);
      int tot_mins = tot_secs / 60;
      int h = tot_mins / 60;
      int m = tot_mins % 60;
      strNextAlarm = tr("Next alarm in ");
      if (h == 1)
      {
        strNextAlarm += tr("1 hour, ");
      }
      else if (h > 1)
      {
        strNextAlarm += tr("%1 hours, ").arg(h);
      }

      if (m == 1)
      {
        strNextAlarm += tr("1 minute");  
      }
      else if (m == 0 && h == 0)
      {
        strNextAlarm += tr("less than 1 minute");
      }
      else
      {
        strNextAlarm += tr("%1 minutes").arg(m);
      }
    }
    else
    {
      strNextAlarm = tr("No alarms scheduled");
    }

    if (txtNextAlarm->text() != strNextAlarm)
    {
      txtNextAlarm->setText(strNextAlarm);
    }
}

void AlarmClock::aboutClicked()
{
    QMessageBox mb(tr("About Alarm Clock"),
         tr("<center><p>Version ") + VERSION +
         tr("<br><br>This program is Copyright (C) 2002 "
         "Dafydd&nbsp;Walters<br>"
         "&lt;dwalters@users.sourceforge.net&gt;<br>"
         "German translation by Peter Hildebrandt<br>"
         "Verison ") + VERSION + tr(" features by Anton Maslovsky<br>"
         "&lt;my-zaurus@narod.ru&gt;<br>"
         "<b>http://my-zaurus.narod.ru</b><br><br>"
         "This program may be modified or "
         "redistributed under the terms of "
         "the GNU&nbsp;General&nbsp;Public&nbsp;License<br>"
         "<b>http://www.gnu.org/licenses/gpl.html</b><br></p></center>"),
         QMessageBox::NoIcon, QMessageBox::Ok, QMessageBox::NoButton,
         QMessageBox::NoButton, this);

    mb.showMaximized();
    mb.exec();
}

void AlarmClock::updateUI()
{
    showAlarm(alarms[0], lblTime1, lblFrequency1, groupAlarm1);
    showAlarm(alarms[1], lblTime2, lblFrequency2, groupAlarm2);
    showAlarm(alarms[2], lblTime3, lblFrequency3, groupAlarm3);
    showAlarm(alarms[3], lblTime4, lblFrequency4, groupAlarm4);
}

void AlarmClock::showAlarm(AlarmDetails* alarmDetails, 
   QLabel* lblTime, QLabel* lblFrequency, QGroupBox* groupAlarm)
{
    groupAlarm->setTitle(buildAlarmText(alarmDetails));      

    if (twelveHourClock)
    {
      if (alarmDetails->time().hour() >= 12)
      {
        int hour = alarmDetails->time().hour() == 12 ? 12 :
                   alarmDetails->time().hour() - 12;
        lblTime->setText(QString::number(hour) + ":" +
                         QString().sprintf("%02d",
                         alarmDetails->time().minute()) +
                         tr(" PM"));
      }
      else
      {
        lblTime->setText(QString::number(alarmDetails->time().hour() == 0 ?
                         12 : alarmDetails->time().hour()) +
                         ":" + QString().sprintf("%02d",
                         alarmDetails->time().minute()) +
                         tr(" AM"));
      }
    }
    else
    {
      lblTime->setText(QString().sprintf("%02d:%02d",
                       alarmDetails->time().hour(),
                       alarmDetails->time().minute()));
    }

    lblFrequency->setText(AlarmDetails::frequencyLabel(alarmDetails->frequency()));
}

void AlarmClock::initTwelveHourClock() 
{
    Config config("qpe");
    config.setGroup("Time");
    twelveHourClock = config.readBoolEntry("AMPM", true);
    weekStartMonday = config.readBoolEntry("MONDAY", false);
}

QString AlarmClock::buildAlarmText(AlarmDetails* alarm) const
{
    QString title = alarm->message();
    const size_t MAX_LEN = alarm->enabled() ? 37 : 27;

    if(title.isNull())
      title = tr("Alarm");

    if (title.length() > MAX_LEN)
    {
      title.truncate(MAX_LEN);
      title.append("...");
    }
        
    if (!alarm->enabled())
       title += tr(" (disabled)");

    return title;

}

void AlarmClock::appMessage(const QCString& msg, const QByteArray& data)
{
    int alarmNumber;
    QDataStream ds(data, IO_ReadOnly);
    if (msg == "alarm(QDateTime,int)") 
    {
      QDateTime alarmDateTime;
      ds >> alarmDateTime >> alarmNumber;

      alarms[alarmNumber]->setAlarm(alarms[alarmNumber]->frequency() != AlarmDetails::once,
                                    alarms[alarmNumber]->time(),
                                    alarms[alarmNumber]->frequency(),
                                    alarms[alarmNumber]->message(),
                                    alarms[alarmNumber]->notification(),
                                    alarms[alarmNumber]->led(),
                                    alarms[alarmNumber]->sound(),
                                    alarms[alarmNumber]->device(),         
                                    alarms[alarmNumber]->snoozeInterval(),
                                    alarms[alarmNumber]->weekdaysFlags());
                                    
      AlarmMessage dlg(alarms[alarmNumber], this, "Alarm", true);
      dlg.showMaximized();
      dlg.exec();

	  QCopEnvelope e("QPE/clockApplet", "alarmsChanged()");
	  updateUI();

      // exit after alarm 
      //qApp->closeAllWindows();
 	  //qApp->quit();
	  //exit(0);

    }
	else if (msg == "showAlarms()") 
	{
		updateUI();
	}
	else if (msg == "reloadAlarms()") 
	{
		reloadAlarms();
		updateUI();
	}
	else if (msg == "editAlarm(int)") 
	{
		ds >> alarmNumber;
		editAlarm(alarms[alarmNumber], true); // close applicaiton when done
	}
	else if (msg == "getAlarm(int)") 
	{
	}
}

void AlarmClock::editAlarm(AlarmDetails* alarm, bool bClose /* = false */)
{
    AlarmSet dlg(alarm, this, 0, true);
    dlg.showMaximized();
    dlg.exec();
	updateUI();

	if (bClose)
	{
		qApp->closeAllWindows();
		qApp->quit();
		exit(0);
	}
}

void AlarmClock::alarm1clicked()
{
    editAlarm(alarms[0]);
}

void AlarmClock::alarm2clicked()
{
    editAlarm(alarms[1]);
}

void AlarmClock::alarm3clicked()
{
    editAlarm(alarms[2]);
}

void AlarmClock::alarm4clicked()
{
    editAlarm(alarms[3]);
}
